/*
   código para controlar los diferentes motores del brazo robótico ELISAVA
   Código original Joan Gomez
   Modificado por Alex Latorre y Jonathan Chacón
*/
const int LED1 = A0;
#include <AccelStepper.h>
#include <MultiStepper.h>

const int NUMJOINT = 6;
const int STEPNUM = NUMJOINT + 1;

AccelStepper stepper[STEPNUM];


AccelStepper stepper0(AccelStepper::DRIVER, 11, 10); //DRIVER, CLK, DIR
AccelStepper stepper1(AccelStepper::DRIVER, 12, 13);
AccelStepper stepper2(AccelStepper::DRIVER, 8, 9);
AccelStepper stepper3(AccelStepper::DRIVER, 7, 6);
AccelStepper stepper4(AccelStepper::DRIVER, 5, 4);
AccelStepper stepper5(AccelStepper::DRIVER, 3, 2);
AccelStepper stepper6(AccelStepper::DRIVER, 52, 53);

MultiStepper steppers;

int outputsDT[NUMJOINT]  = {24, 26, 28, 30, 32, 34}; // para los DT de los KY-040
int outputsCLK[NUMJOINT]  = {25, 27, 29, 31, 33, 35}; // para los CLK de los KY-040
int counters[NUMJOINT], aState[NUMJOINT], aLastState[NUMJOINT], twins;

long positions[STEPNUM]; // Array of desired stepper positions

void setup() {
  // put your setup code here, to run once:
  Serial.begin(9600);

  stepper[0] = stepper0;
  stepper[1] = stepper1;
  stepper[2] = stepper2;
  stepper[3] = stepper3;
  stepper[4] = stepper4;
  stepper[5] = stepper5;
  stepper[6] = stepper6;

  for (int i = 0; i < STEPNUM; i++)
  {
    stepper[i].setMaxSpeed(50);
    stepper[i].setAcceleration(100);
    if (i == 3)
    {
      stepper[i].setMaxSpeed(250);
      stepper[i].setAcceleration(250);
    }
    steppers.addStepper(stepper[i]);
  }

  for (int i = 0; i < NUMJOINT; i++) {
    pinMode(outputsDT[i], INPUT); //DT
    pinMode(outputsCLK[i], INPUT); //CLK
    counters[i] = 0;
    aLastState[i] = digitalRead(outputsDT[i]);
  }

  Serial.println("STARTING");
}

void loop()
{
  positions[0] = positions[0] + 5 * upDatePositions(0);
  twins = upDatePositions(1);
  positions[1] = positions[1] + 5 * twins;
  positions[2] = positions[2] - 5 * twins;
  positions[3] = positions[3] + 25 * upDatePositions(2);
  positions[4] = positions[4] + 25 * upDatePositions(3);
  positions[5] = positions[5] + 25 * upDatePositions(4);
  positions[6] = positions[6] + 25 * upDatePositions(5);
  steppers.moveTo(positions);
  while (steppers.run()) {}
}

int upDatePositions(int i) {
  aState[i] = digitalRead(outputsDT[i]);
  int tmpCounter = counters[i];
  int c = 0;
  if (aState[i] != aLastState[i])
  {
    if (digitalRead(outputsCLK[i]) != aState[i]) {
      counters[i] ++;
      c = 1;
    } else {
      counters[i] --;
      c = -1;
    }
    /* Serial.print("Position ");
      Serial.print(i);
      Serial.print(": ");
      Serial.println(counters[i]);
      Serial.println(tmpCounter);
      Serial.println(c * 2);*/
  }
  aLastState[i] = aState[i]; // Guardamos el ultimo valor
  return (c * 2);
}
